package server

import (
	"github.com/odysseus/nodemanager/config"
	"github.com/odysseus/nodemanager/nmregistry"
	"github.com/odysseus/nodemanager/utils"
	omanager "github.com/odysseus/odysseus-protocol/gen/proto/go/nodemanager/v1"
	"github.com/redis/go-redis/v9"
	log "github.com/sirupsen/logrus"
	"google.golang.org/grpc"
	"net"
)

type Node struct {
	registry  *nmregistry.RegistryService
	apiServer *grpc.Server
	rdb       *redis.Client
}

func NewNode() *Node {
	rdb := utils.NewRedisClient(utils.RedisConnParam{
		Addr:     "",
		Password: "",
		DbIndex:  0,
	})
	node := &Node{
		rdb:       rdb,
		apiServer: grpc.NewServer(grpc.MaxSendMsgSize(1024*1024*20), grpc.MaxRecvMsgSize(1024*1024*20)),
		registry:  nmregistry.NewRegistryService(config.GetConfig(), rdb),
	}

	return node
}

func (n *Node) Start() error {
	n.registry.Start()
	if err := n.apiStart(); err != nil {
		return err
	}
	return nil
}

func (n *Node) apiStart() error {
	lis, err := net.Listen("tcp", config.GetConfig().Endpoint)
	if err != nil {
		log.WithError(err).Error("failed to listen endpoint")
		return err
	}

	omanager.RegisterNodeManagerServiceServer(n.apiServer, &NodeManagerService{
		quit: make(chan struct{}),
		node: n,
	})

	err = n.apiServer.Serve(lis)
	if err != nil {
		log.WithError(err).Error("failed to serve apiserver")
		return err
	}
	return nil
}

func (n *Node) Stop() {
	n.registry.Stop()
	n.apiServer.Stop()
}
